﻿using Entidades.Contacto;
using Negocio.Contacto;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ModeloEnCapas
{
    public partial class CrearContacto : Form
    {
        /// <summary>
        /// Obtiene del App.Config la cadena de conexión adecuada para conectarnos a la BD
        /// </summary>
        private string cadenaConexion = ConfigurationManager.ConnectionStrings["CadenaDeConexion"].ToString();

        public CrearContacto()
        {
            InitializeComponent();
        }

        private void textBox_Identificacion_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            Regex reg = new Regex(@"^\d+$");

            if (textBox_Identificacion.Text.Length == 0)
            {
                error = true;
                mensaje = "Este Campo no puede estar vacio.";
            }

            if (reg.IsMatch(textBox_Identificacion.Text) == false)
            {
                error = true;
                mensaje = "Este campo solo puede contener numeros.";
            }

            errorProvider1.SetError(textBox_Identificacion, mensaje);
            
            e.Cancel = error;
        }

        private void textBox_PrimerNombre_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            if (textBox_PrimerNombre.Text.Length == 0)
            {
                error = true;
                mensaje = "Este campo no puede estar vacio.";
            }

            errorProvider1.SetError(textBox_PrimerNombre, mensaje);
            e.Cancel = error;
        }

        private void textBox_PrimerApellido_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            if (textBox_PrimerApellido.Text.Length == 0)
            {
                error = true;
                mensaje = "Este campo no puede estar vacio.";
            }

            errorProvider1.SetError(textBox_PrimerApellido, mensaje);
            e.Cancel = error;
        }

        private void textBox_Correo_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;
            Regex reg = new Regex(@"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*");

            if (textBox_Correo.Text.Length == 0)
            {
                error = true;
                mensaje = "Este Campo no puede estar vacio.";
            }

            if (reg.IsMatch(textBox_Correo.Text) == false)
            {
                error = true;
                mensaje = "El formato del correo suministrado no es correcto. Ejemplo: correo@correo.com";
            }

            errorProvider1.SetError(textBox_Correo, mensaje);
            e.Cancel = error;
        }

        private void textBox_Identificacion_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_Identificacion, string.Empty);
        }

        private void textBox_PrimerNombre_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_PrimerNombre, string.Empty);
        }

        private void textBox_PrimerApellido_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_PrimerApellido, string.Empty);
        }

        private void textBox_Correo_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_Correo, string.Empty);
        }

        private void button_Crear_Click(object sender, EventArgs e)
        {
            if (ValidateChildren() == true)
            {
                EContacto objEContactoCrear = new EContacto();

                objEContactoCrear.Uuid = Guid.NewGuid().ToString();
                
                objEContactoCrear.Identificacion = textBox_Identificacion.Text;
                objEContactoCrear.PrimerNombre = textBox_PrimerNombre.Text;
                objEContactoCrear.SegundoNombre = textBox_SegundoNombre.Text ?? null;
                objEContactoCrear.PrimerApellido = textBox_PrimerApellido.Text;
                objEContactoCrear.SegundoApellido = textBox_SegundoApellido.Text ?? null;
                objEContactoCrear.Correo = textBox_Correo.Text;

                NContacto objNContacto = new NContacto(cadenaConexion);

                EContacto objEContactoFinal = objNContacto.Crear(objEContactoCrear);

                if (objNContacto.TipoMensaje == 1)
                {
                    MessageBox.Show("El contacto fue creado correctamente. Se le asignó el Id: " + objEContactoFinal.Id.ToString() + " y Uuid: " + objEContactoFinal.Uuid);

                    textBox_Identificacion.Text = string.Empty;
                    textBox_PrimerNombre.Text = string.Empty;
                    textBox_SegundoNombre.Text = string.Empty;
                    textBox_PrimerApellido.Text = string.Empty;
                    textBox_SegundoApellido.Text = string.Empty;
                    textBox_Correo.Text = string.Empty;
                }
                else
                {
                    MessageBox.Show(objNContacto.Mensaje);
                }
            }
        }
    }
}
