﻿using Entidades.Contacto;
using Negocio.Contacto;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ModeloEnCapas
{
    public partial class EliminarContacto : Form
    {
        /// <summary>
        /// Obtiene del App.Config la cadena de conexión adecuada para conectarnos a la BD
        /// </summary>
        private string cadenaConexion = ConfigurationManager.ConnectionStrings["CadenaDeConexion"].ToString();

        public EliminarContacto()
        {
            InitializeComponent();
        }

        private void textBox_BuscarIdentificacion_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            Regex reg = new Regex(@"^\d+$");

            if (textBox_BuscarIdentificacion.Text.Length == 0)
            {
                error = true;
                mensaje = "Este Campo no puede estar vacio.";
            }

            if (reg.IsMatch(textBox_BuscarIdentificacion.Text) == false)
            {
                error = true;
                mensaje = "Este campo solo puede contener numeros.";
            }

            errorProvider1.SetError(textBox_BuscarIdentificacion, mensaje);

            e.Cancel = error;
        }

        private void button_Buscar_Click(object sender, EventArgs e)
        {
            if (ValidateChildren(ValidationConstraints.Enabled) == true)
            {
                NContacto objNContacto = new NContacto(cadenaConexion);

                EContacto objEContactoEncontrado = objNContacto.ObtenerPorIdentificacion(textBox_BuscarIdentificacion.Text);

                if (objNContacto.TipoMensaje == 1)
                {
                    // todo ok en negocio
                    if (string.IsNullOrEmpty(objEContactoEncontrado.Uuid) == false)
                    {
                        // si se encontró un contacto.
                        textBox_Id.Text = objEContactoEncontrado.Id.ToString();
                        textBox_Uuid.Text = objEContactoEncontrado.Uuid;
                        textBox_Identificacion.Text = objEContactoEncontrado.Identificacion;
                        textBox_PrimerNombre.Text = objEContactoEncontrado.PrimerNombre;
                        textBox_SegundoNombre.Text = objEContactoEncontrado.SegundoNombre;
                        textBox_PrimerApellido.Text = objEContactoEncontrado.PrimerApellido;
                        textBox_SegundoApellido.Text = objEContactoEncontrado.SegundoApellido;
                        textBox_Correo.Text = objEContactoEncontrado.Correo;

                        button_Cancelar.Enabled = true;
                        button_Eliminar.Enabled = true;

                        textBox_BuscarIdentificacion.Enabled = false;
                        button_Buscar.Enabled = false;
                    }
                    else
                    {
                        MessageBox.Show("No se encontró un Contacto con esa Identificación.");
                    }
                }
                else
                {
                    MessageBox.Show(objNContacto.Mensaje);
                }
            }
        }

        private void EliminarContacto_Load(object sender, EventArgs e)
        {
            flowLayoutPanel1.Enabled = false;
            textBox_BuscarIdentificacion.Text = string.Empty;

            button_Cancelar.Enabled = false;
            button_Eliminar.Enabled = false;
        }

        private void button_Eliminar_Click(object sender, EventArgs e)
        {
            if (ValidateChildren(ValidationConstraints.Enabled) == true)
            {
                NContacto objNContacto = new NContacto(cadenaConexion);

                Int32 filasAfectadas = objNContacto.Eliminar(Convert.ToInt32(textBox_Id.Text), textBox_Uuid.Text);

                if (objNContacto.TipoMensaje == 1)
                {
                    MessageBox.Show("El contácto fue eliminado correctamente. Se procesaron " + filasAfectadas.ToString() + " registros.");

                    textBox_Id.Text = string.Empty;
                    textBox_Uuid.Text = string.Empty;
                    textBox_Identificacion.Text = string.Empty;
                    textBox_PrimerNombre.Text = string.Empty;
                    textBox_SegundoNombre.Text = string.Empty;
                    textBox_PrimerApellido.Text = string.Empty;
                    textBox_SegundoApellido.Text = string.Empty;
                    textBox_Correo.Text = string.Empty;
                    button_Cancelar.Enabled = false;
                    button_Eliminar.Enabled = false;

                    textBox_BuscarIdentificacion.Text = string.Empty;
                    textBox_BuscarIdentificacion.Enabled = true;
                    button_Buscar.Enabled = true;
                }
                else
                {
                    MessageBox.Show(objNContacto.Mensaje);
                }
            }
        }

        private void textBox_BuscarIdentificacion_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_BuscarIdentificacion, string.Empty);
        }
    }
}
