﻿using Entidades.Contacto;
using Negocio.Contacto;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ModeloEnCapas
{
    public partial class ModificarContacto : Form
    {
        /// <summary>
        /// Obtiene del App.Config la cadena de conexión adecuada para conectarnos a la BD
        /// </summary>
        private string cadenaConexion = ConfigurationManager.ConnectionStrings["CadenaDeConexion"].ToString();

        public ModificarContacto()
        {
            InitializeComponent();
        }

        private void ModificarContacto_Load(object sender, EventArgs e)
        {
            flowLayoutPanel1.Enabled = false;
            textBox_BuscarIdentificacion.Text = string.Empty;

            button_Cancelar.Enabled = false;
            button_Modificar.Enabled = false;
        }

        private void button_Buscar_Click(object sender, EventArgs e)
        {
            if (ValidateChildren(ValidationConstraints.Enabled) == true)
            {
                NContacto objNContacto = new NContacto(cadenaConexion);

                EContacto objEContactoEncontrado = objNContacto.ObtenerPorIdentificacion(textBox_BuscarIdentificacion.Text);

                if (objNContacto.TipoMensaje == 1)
                {
                    // todo ok en negocio
                    if (string.IsNullOrEmpty(objEContactoEncontrado.Uuid) == false)
                    {
                        // si se encontró un contacto.
                        textBox_Id.Text = objEContactoEncontrado.Id.ToString();
                        textBox_Uuid.Text = objEContactoEncontrado.Uuid;
                        textBox_Identificacion.Text = objEContactoEncontrado.Identificacion;
                        textBox_PrimerNombre.Text = objEContactoEncontrado.PrimerNombre;
                        textBox_SegundoNombre.Text = objEContactoEncontrado.SegundoNombre;
                        textBox_PrimerApellido.Text = objEContactoEncontrado.PrimerApellido;
                        textBox_SegundoApellido.Text = objEContactoEncontrado.SegundoApellido;
                        textBox_Correo.Text = objEContactoEncontrado.Correo;

                        flowLayoutPanel1.Enabled = true;
                        button_Cancelar.Enabled = true;
                        button_Modificar.Enabled = true;

                        textBox_BuscarIdentificacion.Enabled = false;
                        button_Buscar.Enabled = false;
                    }
                    else
                    {
                        MessageBox.Show("No se encontró un Contacto con esa Identificación.");
                    }
                }
                else
                {
                    MessageBox.Show(objNContacto.Mensaje);
                }
            }
        }

        private void textBox_BuscarIdentificacion_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            Regex reg = new Regex(@"^\d+$");

            if (textBox_BuscarIdentificacion.Text.Length == 0)
            {
                error = true;
                mensaje = "Este Campo no puede estar vacio.";
            }

            if (reg.IsMatch(textBox_BuscarIdentificacion.Text) == false)
            {
                error = true;
                mensaje = "Este campo solo puede contener numeros.";
            }

            errorProvider1.SetError(textBox_BuscarIdentificacion, mensaje);

            e.Cancel = error;
        }

        private void button_Cancelar_Click(object sender, EventArgs e)
        {
            flowLayoutPanel1.Enabled = false;
            textBox_Id.Text = string.Empty;
            textBox_Uuid.Text = string.Empty;
            textBox_Identificacion.Text = string.Empty;
            textBox_PrimerNombre.Text = string.Empty;
            textBox_SegundoNombre.Text = string.Empty;
            textBox_PrimerApellido.Text = string.Empty;
            textBox_SegundoApellido.Text = string.Empty;
            textBox_Correo.Text = string.Empty;
            button_Cancelar.Enabled = false;
            button_Modificar.Enabled = false;

            textBox_BuscarIdentificacion.Enabled = true;
            textBox_BuscarIdentificacion.Text = string.Empty;
            button_Buscar.Enabled = true;
        }

        private void button_Modificar_Click(object sender, EventArgs e)
        {
            if (ValidateChildren(ValidationConstraints.Enabled) == true)
            {
                EContacto objEContactoModificar = new EContacto();

                objEContactoModificar.Id = Convert.ToInt32(textBox_Id.Text);
                objEContactoModificar.Uuid = textBox_Uuid.Text;

                objEContactoModificar.Identificacion = textBox_Identificacion.Text;
                objEContactoModificar.PrimerNombre = textBox_PrimerNombre.Text;
                objEContactoModificar.SegundoNombre = textBox_SegundoNombre.Text ?? null;
                objEContactoModificar.PrimerApellido = textBox_PrimerApellido.Text;
                objEContactoModificar.SegundoApellido = textBox_SegundoApellido.Text ?? null;
                objEContactoModificar.Correo = textBox_Correo.Text;

                NContacto objNContacto = new NContacto(cadenaConexion);

                EContacto objEContactoFinal = objNContacto.Actualizar(objEContactoModificar);

                if (objNContacto.TipoMensaje == 1)
                {
                    MessageBox.Show("El contácto fue modificado correctamente.");

                    flowLayoutPanel1.Enabled = false;
                    textBox_Id.Text = string.Empty;
                    textBox_Uuid.Text = string.Empty;
                    textBox_Identificacion.Text = string.Empty;
                    textBox_PrimerNombre.Text = string.Empty;
                    textBox_SegundoNombre.Text = string.Empty;
                    textBox_PrimerApellido.Text = string.Empty;
                    textBox_SegundoApellido.Text = string.Empty;
                    textBox_Correo.Text = string.Empty;
                    button_Cancelar.Enabled = false;
                    button_Modificar.Enabled = false;

                    textBox_BuscarIdentificacion.Text = string.Empty;
                    textBox_BuscarIdentificacion.Enabled = true;
                    button_Buscar.Enabled = true;
                }
                else
                {
                    MessageBox.Show(objNContacto.Mensaje);
                }
            }
        }

        private void textBox_PrimerNombre_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            if (textBox_PrimerNombre.Text.Length == 0)
            {
                error = true;
                mensaje = "Este campo no puede estar vacio.";
            }

            errorProvider1.SetError(textBox_PrimerNombre, mensaje);
            e.Cancel = error;
        }

        private void textBox_PrimerApellido_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;

            if (textBox_PrimerApellido.Text.Length == 0)
            {
                error = true;
                mensaje = "Este campo no puede estar vacio.";
            }

            errorProvider1.SetError(textBox_PrimerApellido, mensaje);
            e.Cancel = error;
        }

        private void textBox_Correo_Validating(object sender, CancelEventArgs e)
        {
            bool error = false;
            string mensaje = string.Empty;
            Regex reg = new Regex(@"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*");

            if (textBox_Correo.Text.Length == 0)
            {
                error = true;
                mensaje = "Este Campo no puede estar vacio.";
            }

            if (reg.IsMatch(textBox_Correo.Text) == false)
            {
                error = true;
                mensaje = "El formato del correo suministrado no es correcto. Ejemplo: correo@correo.com";
            }

            errorProvider1.SetError(textBox_Correo, mensaje);
            e.Cancel = error;
        }

        private void textBox_BuscarIdentificacion_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_BuscarIdentificacion, string.Empty);
        }

        private void textBox_PrimerNombre_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_PrimerNombre, string.Empty);
        }

        private void textBox_PrimerApellido_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_PrimerApellido, string.Empty);
        }

        private void textBox_Correo_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(textBox_Correo, string.Empty);
        }
    }
}
