﻿using Datos.DatosMySQL;
using Entidades.Contacto;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Negocio.Contacto
{
    /// <summary>
    /// Clase donde almacenamos todas las reglas de negocio relacionadas con la entidad Contacto
    /// </summary>
    public class NContacto
    {
        #region variables y propiedades

        /// <summary>
        /// cadena de conexion a la base de datos
        /// </summary>
        private String cadenadeconexion;

        private Int32 tipomensaje;
        /// <summary>
        /// metodo de comunicación entre capas: 0 = indeterminado, 1 = correcto, 2 = advertencia, 3 = error, 4 = validacion
        /// </summary>
        public Int32 TipoMensaje
        {
            get { return tipomensaje; }
            set { tipomensaje = value; }
        }

        private string mensaje;
        /// <summary>
        /// Mensaje final, complemento de TipoMensaje.
        /// </summary>
        public string Mensaje
        {
            get { return mensaje; }
            set { mensaje = value; }
        }

        #endregion variables y propiedades

        #region constructor

        /// <summary>
        /// Costructor, recibe desde la capa de presentación la cadena de conexion que debemos utilizar para conectar a la base de datos
        /// </summary>
        /// <param name="cadena"></param>
        public NContacto(string cadena)
        {
            cadenadeconexion = cadena;
        }

        #endregion constructor

        #region metodos

        /// <summary>
        /// Obtiene la informacion de un contacto por medio de su identificacion.
        /// </summary>
        /// <param name="identificacion">representa una cadena que contiene el numero de identidad completo del contacto a buscar.</param>
        /// <returns>Entidad Contacto con todos sus atributos con valor.</returns>
        public EContacto ObtenerPorIdentificacion(string identificacion)
        {
            // Entidad contacto a retornar
            EContacto contactoFinal = new EContacto();

            // conexion a la base de datos
            DatosMySQL con = new DatosMySQL(cadenadeconexion);

            // tabla en donde almacenamos la información recibida por la capa de datos.
            DataTable _dtResultado = new DataTable();
            DataRow _drResultado;

            // iniciamos mensaje entre capas
            tipomensaje = 1; // 1 = correcto

            // antes de ponernos a realizar cualquier cosa sobre la base de datos, realizamos las validaciones de negocio
            if (ValidarIdentificacion(identificacion) == false)
            {
                tipomensaje = 4; // 4 = validacion
                mensaje = "VALIDACIÓN: Por favor validar el formato de la identificación a buscar. Acepta solo números.";
            }

            if (tipomensaje == 1)
            {
                // si todo ok hasta el momento pues continuamos con la bd.

                try
                {
                    // iniciamos sesion en la bd
                    con.IniciarSesion();

                    // invocamos metodo Traer tabla de la capa de datos, recordemos que este metodo nos retornará una DataTable
                    // en este caso la DataTable resultado tendrá la información completa del contacto encontrado en la base de datos
                    _dtResultado = con.TraerDataTable("usp_contacto_obtener_por_identificacion",
                        identificacion);
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C001 en -EContacto ObtenerPorIdentificacion(string identificacion)-: " + ex.Message;
                }
                finally
                {
                    // finalizamos la conexion
                    con.FinalizarSesion();
                }
            }

            if (tipomensaje == 1)
            {
                // si todo va bien hasta el momento, tipomensaje = 1 pues nos tomamos la molestia de llenar atributos de la entidad a retornar.

                // obtenemos la primer fila de la tabla resultado (contiene los datos del contacto creado)
                try
                {
                    if (_dtResultado.Rows.Count > 0)
                    {
                        _drResultado = _dtResultado.Rows[0];

                        // asignamos los correspondientes valores a cada atributo del objeto contacto que se retornara.
                        // los datos los extraemos de la fila capturada.
                        contactoFinal.Correo = _drResultado["correo"].ToString();
                        contactoFinal.FechaCreacion = Convert.ToDateTime(_drResultado["fecha_creacion"]);
                        contactoFinal.Id = Convert.ToInt32(_drResultado["id"]);
                        contactoFinal.Identificacion = _drResultado["identificacion"].ToString();
                        contactoFinal.PrimerApellido = _drResultado["primer_apellido"].ToString();
                        contactoFinal.PrimerNombre = _drResultado["primer_nombre"].ToString();
                        contactoFinal.SegundoApellido = _drResultado["segundo_apellido"].ToString();
                        contactoFinal.SegundoNombre = _drResultado["segundo_nombre"].ToString();
                        contactoFinal.Uuid = _drResultado["uuid"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C002 en -EContacto ObtenerPorIdentificacion(string identificacion)-: " + ex.Message;
                }
            }

            // con los atributos con su valor cvorrespondiente podemos retornar el nuevo contacto.
            return contactoFinal;
        }

        /// <summary>
        /// Registra en base de datos un nuevo contacto
        /// </summary>
        /// <param name="contacto">Entidad con valor en todos los atributos necesarios para la creación exitosa del contacto en la base de datos.</param>
        /// <returns>Entidad Contacto final, que fue creada en la base de datos, esta entidad viene con todos sus atributos con valor asignado</returns>
        public EContacto Crear(EContacto contactoRecibido)
        {
            // Entidad contacto a retornar
            EContacto contactoFinal = new EContacto();

            // conexion a la base de datos
            DatosMySQL con = new DatosMySQL(cadenadeconexion);

            // tabla en donde almacenamos la información recibida por la capa de datos.
            DataTable _dtResultado = new DataTable();
            DataRow _drResultado;

            // iniciamos mensaje entre capas
            tipomensaje = 1; // 1 = correcto

            // antes de ponernos a realizar acciones sobre la base de datos pues realizamos validaciones
            if (ValidarContactoCrear(contactoRecibido) == false)
            {
                tipomensaje = 4;
                mensaje = "VALIDACIÓN: La información del contacto no está completa o no cumple con los requerimientos básicos.";
            }

            if (tipomensaje == 1)
            {
                // si todo va ok pues continuamos con la bd
                try
                {
                    // iniciamos sesion en la bd
                    con.IniciarSesion();

                    // invocamos metodo Traer tabla de la capa de datos, recordemos que este metodo nos retornará una DataTable
                    // en este caso la DataTable resultado tendrá la información completa del contacto creado finalmente en la base de datos
                    _dtResultado = con.TraerDataTable("usp_contacto_crear",
                        contactoRecibido.Uuid,
                        contactoRecibido.Identificacion,
                        contactoRecibido.PrimerNombre,
                        contactoRecibido.SegundoNombre,
                        contactoRecibido.PrimerApellido,
                        contactoRecibido.SegundoApellido,
                        contactoRecibido.Correo);
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C001 en -EContacto Crear(EContacto contactoRecibido)-: " + ex.Message;
                }
                finally
                {
                    // finalizamos la conexion
                    con.FinalizarSesion();
                }
            }

            if (tipomensaje == 1)
            {
                // si todo va bien hasta el momento, tipomensaje = 1 pues nos tomamos la molestia de llenar atributos de la entidad a retornar.

                // obtenemos la primer fila de la tabla resultado (contiene los datos del contacto creado)
                try
                {
                    if (_dtResultado.Rows.Count > 0)
                    {
                        _drResultado = _dtResultado.Rows[0];

                        // asignamos los correspondientes valores a cada atributo del objeto contacto que se retornara.
                        // los datos los extraemos de la fila capturada.
                        contactoFinal.Correo = _drResultado["correo"].ToString();
                        contactoFinal.FechaCreacion = Convert.ToDateTime(_drResultado["fecha_creacion"]);
                        contactoFinal.Id = Convert.ToInt32(_drResultado["id"]);
                        contactoFinal.Identificacion = _drResultado["identificacion"].ToString();
                        contactoFinal.PrimerApellido = _drResultado["primer_apellido"].ToString();
                        contactoFinal.PrimerNombre = _drResultado["primer_nombre"].ToString();
                        contactoFinal.SegundoApellido = _drResultado["segundo_apellido"].ToString();
                        contactoFinal.SegundoNombre = _drResultado["segundo_nombre"].ToString();
                        contactoFinal.Uuid = _drResultado["uuid"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C002 en -EContacto Crear(EContacto contactoRecibido)-: " + ex.Message;
                }
            }

            // con los atributos con su valor cvorrespondiente podemos retornar el nuevo contacto.
            return contactoFinal;
        }

        /// <summary>
        /// Actualiza primer y segundo nombre, primer y segundo apellido y el email de un contacto
        /// </summary>
        /// <param name="contactoRecibido">Entidad contacto con los atributos a ser actualizados</param>
        /// <returns>Entidad Contacto con los atributos ya actualizados y toda la informacion completa.</returns>
        public EContacto Actualizar(EContacto contactoRecibido)
        {
            // Entidad contacto a retornar
            EContacto contactoFinal = new EContacto();

            // conexion a la base de datos
            DatosMySQL con = new DatosMySQL(cadenadeconexion);

            // tabla en donde almacenamos la información recibida por la capa de datos.
            DataTable _dtResultado = new DataTable();
            DataRow _drResultado;

            // iniciamos mensaje entre capas
            tipomensaje = 1; // 1 = correcto

            //antes de hacer algo en la base de datos realizamos validaciones minimas
            if (ValidarContactoActualizar(contactoRecibido) == false)
            {
                tipomensaje = 4; // 4 = validacion.
                mensaje = "VALIDACIÓN: La información del contacto a actualizar está incompleta o no cumple los requerimientos básicos.";
            }

            if (tipomensaje == 1)
            {
                // si todo va ok, continuamos con la base de datos
                try
                {
                    // iniciamos sesion en la bd
                    con.IniciarSesion();

                    // invocamos metodo Traer tabla de la capa de datos, recordemos que este metodo nos retornará una DataTable
                    // en este caso la DataTable resultado tendrá la información completa del contacto actualizado finalmente en la base de datos
                    _dtResultado = con.TraerDataTable("usp_contacto_actualizar",
                        contactoRecibido.Id,
                        contactoRecibido.Uuid,
                        contactoRecibido.PrimerNombre,
                        contactoRecibido.SegundoNombre,
                        contactoRecibido.PrimerApellido,
                        contactoRecibido.SegundoApellido,
                        contactoRecibido.Correo);
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C001 en -EContacto Actualizar(EContacto contactoRecibido)-: " + ex.Message;
                }
                finally
                {
                    // finalizamos la conexion
                    con.FinalizarSesion();
                }
            }

            if (tipomensaje == 1)
            {
                // si todo va bien hasta el momento, tipomensaje = 1 pues nos tomamos la molestia de llenar atributos de la entidad a retornar.

                // obtenemos la primer fila de la tabla resultado (contiene los datos del contacto creado)
                try
                {
                    if (_dtResultado.Rows.Count > 0)
                    {
                        _drResultado = _dtResultado.Rows[0];

                        // asignamos los correspondientes valores a cada atributo del objeto contacto que se retornara.
                        // los datos los extraemos de la fila capturada.
                        contactoFinal.Correo = _drResultado["correo"].ToString();
                        contactoFinal.FechaCreacion = Convert.ToDateTime(_drResultado["fecha_creacion"]);
                        contactoFinal.Id = Convert.ToInt32(_drResultado["id"]);
                        contactoFinal.Identificacion = _drResultado["identificacion"].ToString();
                        contactoFinal.PrimerApellido = _drResultado["primer_apellido"].ToString();
                        contactoFinal.PrimerNombre = _drResultado["primer_nombre"].ToString();
                        contactoFinal.SegundoApellido = _drResultado["segundo_apellido"].ToString();
                        contactoFinal.SegundoNombre = _drResultado["segundo_nombre"].ToString();
                        contactoFinal.Uuid = _drResultado["uuid"].ToString();
                    }
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C002 en -EContacto Actualizar(EContacto contactoRecibido)-: " + ex.Message;
                }
            }

            // con los atributos con su valor cvorrespondiente podemos retornar el nuevo contacto.
            return contactoFinal;
        }

        /// <summary>
        /// Elimina un contacto!. No recomendable utilizar este metodo!.
        /// </summary>
        /// <param name="id">nuemro indentificador del contacto en la tabla.</param>
        /// <param name="uuid">identificador unico de 36 caracteres que representa al contacto.</param>
        /// <returns>Entero que representa el numero ed filas afectadas por el procedimiento almacenado.</returns>
        public Int32 Eliminar(Int32 id, string uuid)
        {
            // conexion a la base de datos
            DatosMySQL con = new DatosMySQL(cadenadeconexion);

            // entero que representa el numero de filas afectadas
            Int32 filasAfectadas = 0;

            // iniciamos mensaje entre capas
            tipomensaje = 1; // 1 = correcto

            // realizamos validacion primero
            if (ValidarContactoEliminar(id, uuid) == false)
            {
                tipomensaje = 4; // 4 = validacion.
                mensaje = "VALIDACIÓN: los parametros indicados no cumnplen los requerimientos.";
            }

            if (tipomensaje == 1)
            {
                // si todo ok, continuamos.
                try
                {
                    // iniciamos sesion en la bd
                    con.IniciarSesion();

                    // invocamos metodo ejecutar, recordemos que este metodo nos retorna un intero que representa el nuemro de registros que se afectaron con el procedimiento almacenado.
                    filasAfectadas = con.Ejecutar("usp_contacto_eliminar",
                        id,
                        uuid);
                }
                catch (Exception ex)
                {
                    tipomensaje = 3; // 3 = error!
                    mensaje = "ERROR C001 en -Int32 Eliminar(Int32 id, string uuid)-: " + ex.Message;
                }
                finally
                {
                    // finalizamos la conexion
                    con.FinalizarSesion();
                }
            }

            // con los atributos con su valor cvorrespondiente podemos retornar el nuevo contacto.
            return filasAfectadas;
        }

        // --------------------------------------------- metodos de validacion ------------------------------------------------------ //

        /// <summary>
        /// Valida si la cadena de entrada contiene solo numeros en su interior.
        /// </summary>
        /// <param name="identificacion">cadena de entrada la cual se va a comprobar.</param>
        /// <returns>false = la cadena no contiene solo numeros, true = la cadena contiene solo numeros.</returns>
        private bool ValidarIdentificacion(string identificacion)
        {
            //Expresión que solo acepta números.
            Regex reg = new Regex(@"^\d+$");

            if (reg.IsMatch(identificacion) == false)
            {
                // se retorna falso porque no es una cadena de solo numeros
                return false;
            }
            else
            {
                // se retorna verdadero porque la cadena contiene colo numeros.
                return true;
            }
        }

        /// <summary>
        /// Valida que la cadena de entrada cumpla con el formato de un email
        /// </summary>
        /// <param name="correo">cadena de entrada la cual se va a comprobar.</param>
        /// <returns>false = la cadena no tiene formato correcto, true = la cadena tiene formato correcto.</returns>
        private bool ValidarEmail(string correo)
        {
            //Expresión que solo acepta cadenas de caracteres con formato de email. EJEMPLO: correo@correo.com
            Regex reg = new Regex(@"\w+([-+.']\w+)*@\w+([-.]\w+)*\.\w+([-.]\w+)*"); // 

            if (reg.IsMatch(correo) == false)
            {
                // se retorna falso porque no es una cadena que cumpla las condiciones
                return false;
            }
            else
            {
                // se retorna verdadero porque la cadena cumple
                return true;
            }
        }

        /// <summary>
        /// Valida si la informacion de una entidad Contacto cumple con las condiciones minimas para que pueda ser registrado en la base de datos
        /// </summary>
        /// <param name="contacto">Entidad Contacto a validar.</param>
        /// <returns>falso = La Entidad Contacto no cumple los requerimientos basicos. verdadero = la entidad cumple los requerimientos.</returns>
        private bool ValidarContactoCrear(EContacto contacto)
        {
            // validamos identificacion
            if (ValidarIdentificacion(contacto.Identificacion) == false)
            {
                return false;
            }

            // validamos que el primer nombre no sea nulo
            if (string.IsNullOrEmpty(contacto.PrimerNombre) == true)
            {
                return false;
            }

            // validamos que el primer apellido no sea nulo
            if (string.IsNullOrEmpty(contacto.PrimerApellido) == true)
            {
                return false;
            }

            // validamos formato del correo
            if (ValidarEmail(contacto.Correo) == false)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Valida si la informacion de una entidad Contacto cumple con las condiciones minimas para que pueda ser actualizado en la base de datos
        /// </summary>
        /// <param name="contacto">Entidad Contacto a validar.</param>
        /// <returns>falso = La Entidad Contacto no cumple los requerimientos basicos. verdadero = la entidad cumple los requerimientos.</returns>
        private bool ValidarContactoActualizar(EContacto contacto)
        {
            // validamos que el primer nombre no sea nulo
            if (string.IsNullOrEmpty(contacto.PrimerNombre) == true)
            {
                return false;
            }

            // validamos que el primer apellido no sea nulo
            if (string.IsNullOrEmpty(contacto.PrimerApellido) == true)
            {
                return false;
            }

            // validamos formato del correo
            if (ValidarEmail(contacto.Correo) == false)
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Valida si los datos suministrados cumplen con los requerimeintos mínimos para eliminar un contacto en la base de datos.
        /// </summary>
        /// <param name="id">numero entero que representa el indentificador numerico unico del contacto a eliminar.</param>
        /// <param name="uuid">string de 36 caracteres que representa la identificacion unica del contacto en la base de datos.</param>
        /// <returns>falso = los parametros no cumplen los requerimientos basicos. verdadero = los parametros cumplen los requerimientos.</returns>
        private bool ValidarContactoEliminar(Int32 id, string uuid)
        {
            // nuemro mayor a 0.
            if (id <= 0)
            {
                return false;
            }

            // el identificador no puede ser nulo y ademas debe tener exactamente 36 caracteres.
            if (string.IsNullOrEmpty(uuid) == true || uuid.Length != 36)
            {
                return false;
            }

            return true;
        }

        #endregion metodos
    }
}
